#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2019, 2020 Kairo de Araujo
#
import json
import pytest
from unittest import mock


class MockRequestsResponse:
    def __init__(self, json_data, status_code, headers):
        self._headers = headers
        self.json_data = json_data
        self._status_code = status_code

    def json(self):
        return self.json_data

    @property
    def text(self):
        return json.dumps(self.json_data)

    @property
    def headers(self):
        return self._headers

    @property
    def status_code(self):
        return self._status_code


@pytest.fixture
def requests_session_mock_ok():
    return mock.MagicMock(
        post=mock.MagicMock(return_value=mock.PropertyMock(status_code=200)),
        get=mock.MagicMock(return_value=mock.PropertyMock(status_code=200)),
        options=mock.MagicMock(
            return_value=mock.PropertyMock(status_code=200)
        ),
    )


@pytest.fixture
def requests_session_mock_error():
    return mock.MagicMock(
        post=mock.MagicMock(return_value=mock.PropertyMock(status_code=500)),
        get=mock.MagicMock(return_value=mock.PropertyMock(status_code=500)),
        options=mock.MagicMock(
            return_value=mock.PropertyMock(status_code=500)
        ),
    )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2019, 2020 Kairo de Araujo
#
import pytest
import requests
from unittest import mock

from kodaksmarthome.api import KodakSmartHome
from kodaksmarthome.constants import HTTP_CODE
from tests.conftest import MockRequestsResponse
from tests.json_responses import (
    auth_response,
    devices_response,
    events_response,
)


class TestKodakSmartHome:
    def test_unsupported_region(self):

        with pytest.raises(AttributeError):
            KodakSmartHome("fake_user", "fake_pass", region="BR")


@mock.patch("kodaksmarthome.api.requests")
def test__http_request_methods(mock_requests):

    mocked_response_get_post = MockRequestsResponse(
        {"key": "value"}, HTTP_CODE.OK, {"Content-Type": "application/json"}
    )
    mocked_response_options = MockRequestsResponse(
        None, HTTP_CODE.OK, {"Content-Type": "somethingelse"}
    )

    mock_requests.Session.return_value = mock.MagicMock(
        get=mock.MagicMock(return_value=mocked_response_get_post),
        post=mock.MagicMock(return_value=mocked_response_get_post),
        options=mock.MagicMock(return_value=mocked_response_options),
    )

    test_ksh = KodakSmartHome("fake_user", "fake_pass")

    assert test_ksh._http_request("GET", "http://fake=url") == {"key": "value"}

    assert test_ksh._http_request("POST", "http://fake=url") == {
        "key": "value"
    }

    assert test_ksh._http_request("OPTIONS", "http://fake=url")

    with pytest.raises(TypeError) as exception_msg:
        test_ksh._http_request("INVALID", "http://fake=url")
        assert "Invalid Method INVALID" in str(exception_msg.value)


@mock.patch("kodaksmarthome.api.requests")
def test__http_request_session_exception(mock_requests):

    mock_requests.Session.return_value = mock.MagicMock(
        get=mock.MagicMock(side_effect=requests.exceptions.ConnectionError),
        post=mock.MagicMock(side_effect=requests.exceptions.ConnectionError),
        options=mock.MagicMock(
            side_effect=requests.exceptions.ConnectionError
        ),
    )

    test_ksh = KodakSmartHome("fake_user", "fake_pass")

    with pytest.raises(TypeError):
        test_ksh._http_request("GET", "http://fake=url")


@mock.patch("kodaksmarthome.api.requests")
def test__http_request_200_withou_body_methods_get_post(mock_requests):

    no_response = None
    mocked_response_get_post = MockRequestsResponse(
        no_response, HTTP_CODE.OK, {"Content-Type": "NOT_OK"}
    )

    mock_requests.Session.return_value = mock.MagicMock(
        get=mock.MagicMock(return_value=mocked_response_get_post),
        post=mock.MagicMock(return_value=mocked_response_get_post),
    )

    test_ksh = KodakSmartHome("fake_user", "fake_pass")

    with pytest.raises(TypeError) as exception_msg:
        assert test_ksh._http_request("GET", "http://fake=url")
        assert test_ksh.is_connected is False
        assert "Unexpected response format" in str(exception_msg.value)


@mock.patch("kodaksmarthome.api.requests")
def test__http_request_401_invalid_grant(mock_requests):

    invalid_grant_response = {
        "error": "invalid_grant",
        "error_description": " ... invalid grant ...",
    }
    mocked_response_get_post = MockRequestsResponse(
        invalid_grant_response,
        HTTP_CODE.UNAUTHORIZED,
        {"Content-Type": "application/json"},
    )

    mock_requests.Session.return_value = mock.MagicMock(
        get=mock.MagicMock(return_value=mocked_response_get_post),
        post=mock.MagicMock(return_value=mocked_response_get_post),
    )

    test_ksh = KodakSmartHome("fake_user", "fake_pass")

    with pytest.raises(ConnectionError) as exception_msg:
        assert test_ksh._http_request("GET", "http://fake=url")
        assert test_ksh.is_connected is False
        assert "invalid grant" in str(exception_msg.value)


@mock.patch("kodaksmarthome.api.requests")
def test__http_request_401_auth_error_is_connected(mock_requests):

    invalid_grant_response = {
        "error": {"reason": "authError", "message": "auth error"}
    }
    mocked_response_get_post = MockRequestsResponse(
        invalid_grant_response,
        HTTP_CODE.UNAUTHORIZED,
        {"Content-Type": "application/json"},
    )

    mock_requests.Session.return_value = mock.MagicMock(
        get=mock.MagicMock(return_value=mocked_response_get_post),
        post=mock.MagicMock(return_value=mocked_response_get_post),
    )

    test_ksh = KodakSmartHome("fake_user", "fake_pass")
    test_ksh.is_connected = True

    assert test_ksh._http_request("GET", "http://fake=url")
    assert test_ksh.is_connected is False


@mock.patch("kodaksmarthome.api.requests")
def test__http_request_401_auth_error_is_connected_false(mock_requests):

    auth_error_response = {
        "error": {"reason": "authError", "message": "auth error"}
    }
    mocked_response_get_post = MockRequestsResponse(
        auth_error_response,
        HTTP_CODE.UNAUTHORIZED,
        {"Content-Type": "application/json"},
    )

    mock_requests.Session.return_value = mock.MagicMock(
        get=mock.MagicMock(return_value=mocked_response_get_post),
        post=mock.MagicMock(return_value=mocked_response_get_post),
    )

    test_ksh = KodakSmartHome("fake_user", "fake_pass")
    test_ksh.is_connected = False

    with pytest.raises(ConnectionError) as exception_msg:
        assert test_ksh._http_request("GET", "http://fake=url")
        assert test_ksh.is_connected is False
        assert "auth error" in str(exception_msg.value)


@mock.patch("kodaksmarthome.api.requests")
def test__http_request_401_access_denied_is_connected(mock_requests):

    access_denied_response = {"msg": "Access Denied"}
    mocked_response_get_post = MockRequestsResponse(
        access_denied_response,
        HTTP_CODE.UNAUTHORIZED,
        {"Content-Type": "application/json"},
    )

    mock_requests.Session.return_value = mock.MagicMock(
        get=mock.MagicMock(return_value=mocked_response_get_post),
        post=mock.MagicMock(return_value=mocked_response_get_post),
    )

    test_ksh = KodakSmartHome("fake_user", "fake_pass")
    test_ksh.is_connected = True

    assert test_ksh._http_request("GET", "http://fake=url")
    assert test_ksh.is_connected is False


@mock.patch("kodaksmarthome.api.requests")
def test__http_request_401_access_denied_is_connected_false(mock_requests):

    access_denied_response = {"msg": "Access Denied"}
    mocked_response_get_post = MockRequestsResponse(
        access_denied_response,
        HTTP_CODE.UNAUTHORIZED,
        {"Content-Type": "application/json"},
    )

    mock_requests.Session.return_value = mock.MagicMock(
        get=mock.MagicMock(return_value=mocked_response_get_post),
        post=mock.MagicMock(return_value=mocked_response_get_post),
    )

    test_ksh = KodakSmartHome("fake_user", "fake_pass")
    test_ksh.is_connected = False

    with pytest.raises(ConnectionError) as exception_msg:
        assert test_ksh._http_request("GET", "http://fake=url")
        assert test_ksh.is_connected is False
        assert "Access Denied" in str(exception_msg.value)


@mock.patch("kodaksmarthome.api.requests")
def test__http_request_401_unexpected(mock_requests):

    unexpected_response = {"not_mapped": "Strange Error"}
    mocked_response_get_post = MockRequestsResponse(
        unexpected_response,
        HTTP_CODE.UNAUTHORIZED,
        {"Content-Type": "application/json"},
    )

    mock_requests.Session.return_value = mock.MagicMock(
        get=mock.MagicMock(return_value=mocked_response_get_post),
        post=mock.MagicMock(return_value=mocked_response_get_post),
    )

    test_ksh = KodakSmartHome("fake_user", "fake_pass")
    test_ksh.is_connected = True

    with pytest.raises(ConnectionError) as exception_msg:
        assert test_ksh._http_request("GET", "http://fake=url")
        assert test_ksh.is_connected is False
        assert "Unexpected 401 error" in str(exception_msg.value)


@mock.patch("kodaksmarthome.api.requests")
def test__http_request_not_200_401_http_code(mock_requests):

    mocked_response_get_post = MockRequestsResponse(
        None, HTTP_CODE.INTERNAL_SERVER_ERROR, "None"
    )

    mock_requests.Session.return_value = mock.MagicMock(
        get=mock.MagicMock(return_value=mocked_response_get_post),
        post=mock.MagicMock(return_value=mocked_response_get_post),
    )

    test_ksh = KodakSmartHome("fake_user", "fake_pass")
    test_ksh.is_connected = True

    with pytest.raises(ConnectionError) as exception_msg:
        assert test_ksh._http_request("GET", "http://fake=url")
        assert test_ksh.is_connected is False
        assert "Unexpected HTTP CODE error" in str(exception_msg.value)


@mock.patch("kodaksmarthome.api.KodakSmartHome._http_request")
def test__options(mock__http_request):

    mock__http_request.return_value = True
    test_ksh = KodakSmartHome("fake_user", "fake_pass")

    assert test_ksh._options()


@mock.patch("kodaksmarthome.api.KodakSmartHome._http_request")
def test__options_exception(mock__http_request):

    mock__http_request.side_effect = [ConnectionError]
    test_ksh = KodakSmartHome("fake_user", "fake_pass")

    with pytest.raises(ConnectionError):
        test_ksh._options()


@mock.patch("kodaksmarthome.api.KodakSmartHome._http_request")
def test__token(mock__http_request):

    mock__http_request.return_value = {
        "access_token": "access_token",
        "token_type": "token_type",
        "refresh_token": "refresh_token",
        "expires_in": "expires_in",
        "scope": "scope",
        "account_info": "account_info",
        "web_urls": "web_urls",
    }

    test_ksh = KodakSmartHome("fake_user", "fake_pass")

    assert test_ksh._token()
    assert test_ksh.token == "access_token"


@mock.patch("kodaksmarthome.api.KodakSmartHome._http_request")
def test__get_token_exception(mock__http_request):

    mock__http_request.side_effect = [ConnectionError]
    test_ksh = KodakSmartHome("fake_user", "fake_pass")

    with pytest.raises(ConnectionError):
        test_ksh._token()


@mock.patch("kodaksmarthome.api.requests")
@mock.patch("kodaksmarthome.api.KodakSmartHome._http_request")
def test__authentication(mock_requests, mock__http_request):
    mocked_response = MockRequestsResponse(
        devices_response, HTTP_CODE.OK, None
    )

    mock__http_request.return_value = auth_response
    mock_requests.Session.return_value = mock.MagicMock(
        get=mock.MagicMock(return_value=mocked_response)
    )

    test_ksh = KodakSmartHome("fake_user", "fake_pass")

    assert test_ksh._authentication()


@mock.patch("kodaksmarthome.api.KodakSmartHome._http_request")
def test__authentication_exception(mock__http_request):
    mock__http_request.side_effect = [ConnectionError]
    test_ksh = KodakSmartHome("fake_user", "fake_pass")

    with pytest.raises(ConnectionError):
        test_ksh._authentication()


@mock.patch("kodaksmarthome.api.KodakSmartHome._http_request")
def test__get_devices(mock__http_request):

    mock__http_request.return_value = devices_response
    test_ksh = KodakSmartHome("fake_user", "fake_pass")
    test_ksh.is_connected = True

    test_devices = test_ksh._get_devices()

    assert test_devices == devices_response["data"]


@mock.patch("kodaksmarthome.api.KodakSmartHome._http_request")
@mock.patch("kodaksmarthome.api.KodakSmartHome.connect")
def test__get_devices_is_connected_false(mock__http_request, mock_connect):

    mock__http_request.return_value = [devices_response, devices_response]
    mock_connect.return_value = devices_response
    test_ksh = KodakSmartHome("fake_user", "fake_pass")
    test_ksh.is_connected = False

    test_devices = test_ksh._get_devices()

    assert test_devices == []


@mock.patch("kodaksmarthome.api.KodakSmartHome._http_request")
def test__get_devices_exception(mock__http_request):
    mock__http_request.side_effect = [ConnectionError]
    test_ksh = KodakSmartHome("fake_user", "fake_pass")
    test_ksh.is_connected = True

    with pytest.raises(ConnectionError):
        test_ksh._get_devices()


@mock.patch("kodaksmarthome.api.KodakSmartHome._http_request")
def test__get_events(mock__http_request):

    mock__http_request.return_value = events_response
    test_ksh = KodakSmartHome("fake_user", "fake_pass")
    test_ksh.devices = devices_response["data"]["devices"]
    test_ksh.is_connected = True
    test_events = test_ksh._get_events()

    expected_result = [
        {
            "device_id": devices_response["data"]["devices"][0]["device_id"],
            "events": events_response["data"]["events"],
        }
    ]

    assert test_events == expected_result


@mock.patch("kodaksmarthome.api.KodakSmartHome._http_request")
@mock.patch("kodaksmarthome.api.KodakSmartHome.connect")
def test__get_events_is_connected_false(mock__http_request, mock_connect):

    mock__http_request.return_value = events_response
    mock_connect.return_value = True
    test_ksh = KodakSmartHome("fake_user", "fake_pass")
    test_ksh.devices = devices_response["data"]["devices"]
    test_ksh.is_connected = False
    test_events = test_ksh._get_events()

    assert test_events == [{"device_id": "FAKEDEVICEID", "events": []}]


@mock.patch("kodaksmarthome.api.KodakSmartHome._http_request")
def test__get_events_none(mock__http_request):
    events_response_none = {
        "status": HTTP_CODE.OK,
        "msg": "Success",
        "total_pages": 0,
        "data": {"total_events": 0, "total_pages": 0, "events": []},
    }

    mock__http_request.return_value = events_response_none

    test_ksh = KodakSmartHome("fake_user", "fake_pass")
    test_ksh.devices = devices_response["data"]["devices"]
    test_ksh.token = "abcdef0123456789"
    test_ksh.is_connected = True
    test_events = test_ksh._get_events()

    expected_result = [
        {
            "device_id": devices_response["data"]["devices"][0]["device_id"],
            "events": events_response_none["data"]["events"],
        }
    ]

    assert test_events == expected_result


@mock.patch("kodaksmarthome.api.KodakSmartHome._http_request")
def test__get_events_exception(mock__http_request):

    mock__http_request.side_effect = [ConnectionError]
    test_ksh = KodakSmartHome("fake_user", "fake_pass")
    test_ksh.devices = devices_response["data"]["devices"]
    test_ksh.token = "abcdef0123456789"

    with pytest.raises(ConnectionError):
        test_ksh._get_events()


@mock.patch("kodaksmarthome.api.KodakSmartHome._options")
@mock.patch("kodaksmarthome.api.KodakSmartHome._token")
@mock.patch("kodaksmarthome.api.KodakSmartHome._authentication")
@mock.patch("kodaksmarthome.api.KodakSmartHome._get_devices")
@mock.patch("kodaksmarthome.api.KodakSmartHome._get_events")
def test_connect(
    mock__options,
    mock__token,
    mock__authentication,
    mock__get_devices,
    mock__get_events,
):
    mock__options.return_value = True
    mock__token.return_value = True
    mock__authentication.return_value = True
    mock__get_devices.return_value = True
    mock__get_events.return_value = True

    test_ksh = KodakSmartHome("fake_user", "fake_pass")

    assert test_ksh.connect() is None


@mock.patch("kodaksmarthome.api.KodakSmartHome._options")
def test_connect_exception(mock__options,):
    mock__options.side_effect = requests.exceptions.ConnectionError
    test_ksh = KodakSmartHome("fake_user", "fake_pass")

    with pytest.raises(ConnectionError):
        test_ksh.connect()


@mock.patch("kodaksmarthome.api.KodakSmartHome._http_request")
def test_disconnect(mock__http_request):

    mock__http_request.return_value = True
    test_ksh = KodakSmartHome("fake_user", "fake_pass")
    test_ksh.disconnect()

    assert test_ksh.is_connected is False


@mock.patch("kodaksmarthome.api.KodakSmartHome._get_devices")
@mock.patch("kodaksmarthome.api.KodakSmartHome._get_events")
def test_update(mock__get_devices, mock__get_events):
    mock__get_devices.return_value = True
    mock__get_events.return_value = True

    test_ksh = KodakSmartHome("fake_user", "fake_pass")

    assert test_ksh.update() is None


def test_get_devices():
    test_ksh = KodakSmartHome("fake_user", "fake_pass")
    test_ksh.is_connected = True
    test_ksh.devices = devices_response["data"]

    assert test_ksh.get_devices == devices_response["data"]


def test_get_devices_disconnected():
    test_ksh = KodakSmartHome("fake_user", "fake_pass")

    with pytest.raises(ConnectionError):
        test_ksh.get_devices


def test_get_events():
    test_ksh = KodakSmartHome("fake_user", "fake_pass")
    test_ksh.is_connected = True
    test_ksh.devices = devices_response["data"]
    test_ksh.events = events_response["data"]["events"]

    assert test_ksh.get_events == events_response["data"]["events"]


def test_get_events_disconnected():
    test_ksh = KodakSmartHome("fake_user", "fake_pass")
    test_ksh.is_connected = False

    with pytest.raises(ConnectionError):
        test_ksh.get_events


def test__filter_event_type():
    test_ksh = KodakSmartHome("fake_user", "fake_pass")
    test_ksh.is_connected = True
    test_ksh.events = [
        {
            "device_id": devices_response["data"]["devices"][0]["device_id"],
            "events": events_response["data"]["events"],
        }
    ]
    test_ksh.devices = devices_response["data"]["devices"]

    test_result = test_ksh._filter_event_type(device_id="FAKEDEVICEID")

    assert len(test_result) == 2


def test__filter_event_type_none():
    test_ksh = KodakSmartHome("fake_user", "fake_pass")
    test_ksh.is_connected = True
    test_ksh.events = [
        {
            "device_id": devices_response["data"]["devices"][0]["device_id"],
            "events": events_response["data"]["events"],
        }
    ]
    test_ksh.devices = devices_response["data"]["devices"]

    test_result = test_ksh._filter_event_type(device_id=None)

    assert len(test_result) == 2


def test__filter_event_type_disconnected():
    test_ksh = KodakSmartHome("fake_user", "fake_pass")
    test_ksh.is_connected = False

    with pytest.raises(ConnectionError):
        test_ksh._filter_event_type(device_id=None)


def test_get_motion_events():
    test_ksh = KodakSmartHome("fake_user", "fake_pass")
    test_ksh.is_connected = True
    test_ksh.events = [
        {
            "device_id": devices_response["data"]["devices"][0]["device_id"],
            "events": events_response["data"]["events"],
        }
    ]
    test_ksh.devices = devices_response["data"]["devices"]

    assert len(test_ksh.get_motion_events(device_id="FAKEDEVICEID")) == 2


def test_get_motion_events_invalid_device_id():
    test_ksh = KodakSmartHome("fake_user", "fake_pass")
    test_ksh.is_connected = True
    test_ksh.events = [
        {
            "device_id": devices_response["data"]["devices"][0]["device_id"],
            "events": events_response["data"]["events"],
        }
    ]
    test_ksh.devices = devices_response["data"]["devices"]

    assert len(test_ksh.get_motion_events(device_id="INVALID")) == 0


def test_get_motion_events_disconnected():
    test_ksh = KodakSmartHome("fake_user", "fake_pass")

    with pytest.raises(ConnectionError):
        test_ksh.get_motion_events()


def test_get_battery_events():
    test_ksh = KodakSmartHome("fake_user", "fake_pass")
    test_ksh.is_connected = True
    test_ksh.events = [
        {
            "device_id": devices_response["data"]["devices"][0]["device_id"],
            "events": events_response["data"]["events"],
        }
    ]
    test_ksh.devices = devices_response["data"]["devices"]

    assert len(test_ksh.get_battery_events(device_id="FAKEDEVICEID")) == 3


def test_get_battery_events_disconnected():
    test_ksh = KodakSmartHome("fake_user", "fake_pass")

    with pytest.raises(ConnectionError):
        test_ksh.get_battery_events()


def test_get_battery_events_invalid_device_id():
    test_ksh = KodakSmartHome("fake_user", "fake_pass")
    test_ksh.is_connected = True
    test_ksh.events = [
        {
            "device_id": devices_response["data"]["devices"][0]["device_id"],
            "events": events_response["data"]["events"],
        }
    ]
    test_ksh.devices = devices_response["data"]["devices"]

    assert len(test_ksh.get_battery_events(device_id="INVALID")) == 0


def test_get_sound_events():
    test_ksh = KodakSmartHome("fake_user", "fake_pass")
    test_ksh.is_connected = True
    test_ksh.events = [
        {
            "device_id": devices_response["data"]["devices"][0]["device_id"],
            "events": events_response["data"]["events"],
        }
    ]
    test_ksh.devices = devices_response["data"]["devices"]

    assert len(test_ksh.get_sound_events(device_id="FAKEDEVICEID")) == 1


def test_get_sound_events_disconnected():
    test_ksh = KodakSmartHome("fake_user", "fake_pass")

    with pytest.raises(ConnectionError):
        test_ksh.get_sound_events()


def test_get_sound_events_invalid_device_id():
    test_ksh = KodakSmartHome("fake_user", "fake_pass")
    test_ksh.is_connected = True
    test_ksh.events = [
        {
            "device_id": devices_response["data"]["devices"][0]["device_id"],
            "events": events_response["data"]["events"],
        }
    ]
    test_ksh.devices = devices_response["data"]["devices"]

    assert len(test_ksh.get_sound_events(device_id="INVALID")) == 0



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2019, Kairo de Araujo
#

auth_response = {
    "status": 200,
    "msg": "Success",
    "data": {
        "id": 7777,
        "email": "my@email.com",
        "phone_number": None,
        "user_name": "nickname",
        "first_name": "Name",
        "last_name": "Lastname",
        "status": 1,
        "policy_agreed": None,
        "send_limit_upload_enable": True,
        "date_created": "2019-07-05T14:15:53Z",
        "last_updated": "2019-12-17T19:49:06Z",
        "last_sign_in": "2019-12-17T19:49:06Z",
        "last_sign_in_count": None,
        "tandc_id": 4,
        "current_tandc_id": 4,
        "is_marketing_accepted": True,
    },
}

devices_response = {
    "status": 200,
    "msg": "Success",
    "data": {
        "payment_source": 1,
        "plan_id": "plan_01",
        "next_plan_id": None,
        "plan_status": 1,
        "plan_expiry_date": None,
        "plan_next_bill_date": None,
        "web_sub_url": "https://app-eu.kodaksmarthome.com/web#...",
        "web_conn_sub_url": "https://app-eu.kodaksmarthome.com/web#/...",
        "devices": [
            {
                "id": 1000,
                "device_id": "FAKEDEVICEID",
                "plan_id": "plan_01",
                "web_conn_sub_url_by_device": "https://app-eu.kodaksmarth...",
            }
        ],
    },
}

events_response = {
    "status": 200,
    "msg": "Success",
    "total_pages": 1,
    "data": {
        "total_events": 6,
        "total_pages": 1,
        "events": [
            {
                "id": "13889a30-227c-11ea-8790-3fdecad07f1c",
                "event_type": 7,
                "created_date": "2019-12-19T16:24:48.000Z",
                "data": [],
                "dv_data": None,
            },
            {
                "id": "45d55ee0-2279-11ea-8790-3fdecad07f1c",
                "event_type": 7,
                "created_date": "2019-12-19T16:04:44.000Z",
                "data": [],
                "dv_data": None,
            },
            {
                "id": "1095e3e0-2278-11ea-8790-3fdecad07f1c",
                "event_type": 1,
                "created_date": "2019-12-19T15:56:05.000Z",
                "snapshot": "http://snapshot_url",
                "data": [
                    {
                        "file": "http://file_type2_url",
                        "file_type": 2,
                        "storage_id": 2,
                        "id": "3525c8b1-2278-11ea-8790-3fdecad07f1c",
                        "created_date": "2019-12-19T15:57:07.000Z",
                        "file_size": 2574730,
                    },
                    {
                        "file": "http://file_type1_url",
                        "file_type": 1,
                        "storage_id": 2,
                        "id": "154ab961-2278-11ea-8790-3fdecad07f1c",
                        "created_date": "2019-12-19T15:56:13.000Z",
                        "file_size": 308288,
                    },
                ],
                "dv_data": None,
            },
            {
                "id": "ddaac3b0-2277-11ea-8790-3fdecad07f1c",
                "event_type": 7,
                "created_date": "2019-12-19T15:54:40.000Z",
                "data": [],
                "dv_data": None,
            },
            {
                "id": "ab7c3cc0-2277-11ea-8790-3fdecad07f1c",
                "event_type": 1,
                "created_date": "2019-12-19T15:53:16.000Z",
                "snapshot": "http://snapshot_url",
                "data": [
                    {
                        "file": "http://file_type2_url",
                        "file_type": 2,
                        "storage_id": 2,
                        "id": "c6b08051-2277-11ea-8790-3fdecad07f1c",
                        "created_date": "2019-12-19T15:54:01.000Z",
                        "file_size": 1152934,
                    },
                    {
                        "file": "http://file_type1_url",
                        "file_type": 1,
                        "storage_id": 2,
                        "id": "ae4b8961-2277-11ea-8790-3fdecad07f1c",
                        "created_date": "2019-12-19T15:53:20.000Z",
                        "file_size": 71552,
                    },
                ],
                "dv_data": None,
            },
            {
                "id": "c71516a0-2f3d-11ea-8790-3fdecad07f1c",
                "event_type": 2,
                "created_date": "2020-01-04T22:01:36.000Z",
                "data": [],
                "dv_data": None,
            },
        ],
    },
}



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2019, 2020 Kairo de Araujo
#
from requests.status_codes import codes

# DEVICES EVENTS
DEVICE_EVENT_MOTION = 1
DEVICE_EVENT_SOUND = 2
DEVICE_EVENT_BATTERY = 7

# REGION_URLS
SUPPORTED_REGIONS = {
    "EU": {
        "URL": "https://app-eu.kodaksmarthome.com/web",
        "URL_TOKEN": "https://api-t01-r3.perimetersafe.com/v1/oauth/token",
        "URL_AUTH": "https://app-eu.kodaksmarthome.com/web/authenticate",
        "URL_DEVICES": "https://app-eu.kodaksmarthome.com/web/user/device",
        "URL_LOGOUT": "https://app-eu.kodaksmarthome.com/web/#/user/logout",
    },
    "US": {
        "URL": "https://app-us.kodaksmarthome.com/web",
        "URL_TOKEN": "https://api-t01-r3.perimetersafe.com/v1/oauth/token",
        "URL_AUTH": "https://app-us.kodaksmarthome.com/web/authenticate",
        "URL_DEVICES": "https://app-us.kodaksmarthome.com/web/user/device",
        "URL_LOGOUT": "https://app-us.kodaksmarthome.com/web/#/user/logout",
    },
    "HK": {
        "URL": "https://app-hk.kodaksmarthome.com/web",
        "URL_TOKEN": "https://api-t01-r3.perimetersafe.com/v1/oauth/token",
        "URL_AUTH": "https://app-hk.kodaksmarthome.com/web/authenticate",
        "URL_DEVICES": "https://app-hk.kodaksmarthome.com/web/user/device",
        "URL_LOGOUT": "https://app-hk.kodaksmarthome.com/web/#/user/logout",
    },
    "IN": {
        "URL": "https://app-r1.kodaksmarthome.com/web",
        "URL_TOKEN": "https://api-t01-r1.perimetersafe.com/v1/oauth/token",
        "URL_AUTH": "https://app-r1.kodaksmarthome.com/web/authenticate",
        "URL_DEVICES": "https://app-r1.kodaksmarthome.com/web/user/device",
        "URL_LOGOUT": "https://app-r1.kodaksmarthome.com/web/#/user/logout",
    }
}


class _URLS(object):
    def __init__(self, region):
        self.__dict__ = region


# HTTP General
HTTP_CODE = codes

# HTTP_CLIENT
HTTP_CLIENT_MODEL = (
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_14_6) AppleWebKit/537.36 "
    + "(KHTML, like Gecko) Chrome/79.0.3945.79 Safari/537.36"
)
HTTP_CLIENT_AUTHORIZATION = (
    "Basic MjFmOTk1M2VlZGE4N2I3MGRjMTE1ZTUyNDU2ODE1OWNjNmExNzI2MTNiOGUyMGMwMT"
    + "UzMGZjNjg2ODc3Mzk2ZDo0ZDA5YmZlMWRhMjU0YmRjNzA4YjEzMGIxMzVmYzA2NjU4ODI2"
    + "MWZjNTY2YWQzMWEyMGM1YjA5ZTY3NTFkNTgy"
)


# HTTP_HEADERS
HTTP_HEADERS_BASIC = {
    "Accept": "application/json, text/plain, */*",
    "Accept-Encoding": "gzip, deflate, br",
    "Accept-Language": "en-GB,en-US;q=0.9,en;q=0.8",
    "Cache-Control": "no-cache",
    "Connection": "keep-alive",
    "Sec-Fetch-Site": "cross-site",
    "Sec-Fetch-Mode": "cors",
    "User-Agent": HTTP_CLIENT_MODEL,
}
HTTP_HEADERS_AUTH = {
    **HTTP_HEADERS_BASIC,
    **{"Content-Type": "application/x-www-form-urlencoded"},
    **{"Authorization": HTTP_CLIENT_AUTHORIZATION},
}
HTTP_HEADERS_OPTIONS = {
    "Access-Control-Request-Method": "POST",
    "Access-Control-Request-Headers": "authorization",
    "Authorization": None,
}



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2019 Kairo de Araujo
#
import requests

from kodaksmarthome.constants import (
    HTTP_HEADERS_AUTH,
    HTTP_HEADERS_BASIC,
    HTTP_CODE,
    HTTP_CLIENT_MODEL,
    DEVICE_EVENT_BATTERY,
    DEVICE_EVENT_SOUND,
    DEVICE_EVENT_MOTION,
    SUPPORTED_REGIONS,
    _URLS,
)


class KodakSmartHome:
    """Kodak Smart Home API session.

    Provides connection to Kodak Smart Home portal.

    :param username: username registered in Kodak Smart Home Portal
    :type username: str
    :param password: password registered in Kodak Smart Home Portal
    :type password: str
    :param region: Global Region Portal. Options: 'EU'. Default: 'EU'
    :type region: str
    """

    def __init__(self, username, password, region="EU"):

        self.username = username
        self.password = password
        self.http_session = requests.Session()
        self.token = None
        self.account_info = None
        self.web_urls = None
        self.devices = list()
        self.events = list()
        self.is_connected = False
        if region not in SUPPORTED_REGIONS:
            raise AttributeError(f"{region} is not supported")

        else:
            self.region_url = _URLS(SUPPORTED_REGIONS[region])
            referer = self.region_url.URL.split("/web")[0]
            HTTP_HEADERS_BASIC["Origin"] = self.region_url.URL
            HTTP_HEADERS_BASIC["Referer"] = referer
            self.basic_headers = HTTP_HEADERS_BASIC

    def _http_request(self, method, url, headers=None, data=None, params=None):

        try:
            if method == "POST":
                http_response = self.http_session.post(
                    url, headers=headers, data=data, params=params
                )

            elif method == "OPTIONS":
                http_response = self.http_session.options(
                    url, headers=headers, data=data, params=params
                )

            elif method == "GET":
                http_response = self.http_session.get(
                    url, headers=headers, data=data, params=params
                )

            else:
                raise AttributeError(f"Invalid Method {method}")

        except requests.exceptions.ConnectionError as err:
            raise ConnectionError(str(err))

        status_code = http_response.status_code
        content_type = None
        response_json = None
        response_text = http_response.text
        error = None
        error_description = None

        if "Content-Type" in http_response.headers:
            content_type = http_response.headers["Content-Type"]

        if content_type and "application/json" in content_type:
            response_json = http_response.json()

            if "error" in response_json:
                error = response_json["error"]

            if "error_description" in response_json:
                error_description = response_json["error_description"]

        if status_code == HTTP_CODE.OK:
            if response_json:
                self.is_connected = True
                return response_json

            elif response_json is None and method == "OPTIONS":
                return True

            else:
                self.is_connected = False
                raise TypeError("Unexpected response format")

        elif status_code == HTTP_CODE.UNAUTHORIZED:
            if error == "invalid_grant":
                self.is_connected = False

                raise ConnectionError(error_description)

            elif (
                type(error) == dict
                and "reason" in error
                and error["reason"] == "authError"
                and self.is_connected
            ):
                self.is_connected = False
                return True

            elif (
                type(error) == dict
                and "reason" in error
                and error["reason"] == "authError"
                and self.is_connected is False
            ):
                if "message" in error:
                    raise ConnectionError(error["message"])

            elif (
                "msg" in response_json
                and "Access Denied" in response_json["msg"]
                and self.is_connected
            ):
                self.is_connected = False
                return True

            elif (
                "msg" in response_json
                and "Access Denied" in response_json["msg"]
                and self.is_connected is False
            ):
                self.is_connected = False
                raise ConnectionError(response_json["msg"])

            else:
                self.is_connected = False

                raise ConnectionError("Unexpected 401 error " + response_text)

        else:
            self.is_connected = False
            raise ConnectionError(
                "Unexpected HTTP CODE error " + response_text
            )

    def _options(self):
        """
        Verify the connection with Kodak Smart Home portal

        :return: boolean result
        :rtype: bool
        """
        options_response = self._http_request(
            "OPTIONS", self.region_url.URL_TOKEN, headers=self.basic_headers
        )

        return options_response

    def _token(self):
        """
        Get Kodak Smart Home Portal Token

        :return: True or Raises ``ConnectionError``
        :rtype: bool
        :exception: ``ConnectionError``
        """
        self.token_info = {
            "access_token": None,
            "token_type": None,
            "refresh_token": None,
            "expires_in": None,
            "scope": None,
        }

        token_payload = (
            "grant_type=password&"
            + f"username={self.username}&"
            + f"password={self.password}&"
            + f"model={HTTP_CLIENT_MODEL}"
        )

        token_response = self._http_request(
            "POST",
            self.region_url.URL_TOKEN,
            headers=HTTP_HEADERS_AUTH,
            data=token_payload,
        )

        self.token_info["access_token"] = token_response["access_token"]
        self.token_info["token_type"] = token_response["token_type"]
        self.token_info["refresh_token"] = token_response["refresh_token"]
        self.token_info["expires_in"] = token_response["expires_in"]
        self.token_info["scope"] = token_response["scope"]
        self.account_info = token_response["account_info"]
        self.web_urls = token_response["web_urls"]
        self.token_info["access_token"] = token_response["access_token"]
        self.token = self.token_info["access_token"]

        return True

    def _authentication(self):
        """
        Perform authentication to Kodak Smart Home Portal

        :return: True or Raises ``ConnectionError``
        :rtype: bool
        """
        auth_payload = f"username=&password={self.token}&rememberme=false"
        auth_response = self._http_request(
            "POST",
            self.region_url.URL_AUTH,
            headers=HTTP_HEADERS_AUTH,
            data=auth_payload,
        )

        self.cookie = self.http_session.cookies["JSESSIONID"]
        self.user_id = auth_response["data"]["id"]

        return True

    def _get_devices(self):
        """
        Get all devices available in Kodak Smart Home Portal

        :return: all devices
        :rtype: list
        """

        headers = self.basic_headers
        headers["Authorization"] = f"Bearer {self.token}"

        devices_response = self._http_request(
            "GET",
            self.region_url.URL_DEVICES,
            headers=headers,
        )

        if self.is_connected is False:
            self.connect()

            return self.devices

        else:
            self.devices = devices_response["data"]

        return self.devices

    def _get_events(self):
        """
        Get all event for all available devices in Kodak Smart Home Portal

        :return: all events
        :rtype: list
        """

        headers = self.basic_headers
        headers["Authorization"] = f"Bearer {self.token}"

        self.events = list()
        for device in self.devices:
            device_id = device["device_id"]
            device_events = {"device_id": device_id, "events": list()}
            pages = 1
            events_pages = 1
            while pages <= events_pages:
                url_events = (
                    f"{self.region_url.URL}/user/device/event?"
                    + f"deviceId={device_id}&"
                    + f"page={pages}"
                )

                events_response = self._http_request(
                    "GET", url_events, headers=headers
                )

                if self.is_connected is False:
                    self.connect()
                    break

                events_pages = events_response["data"]["total_pages"]
                if events_response["data"]["total_events"] == 0:
                    continue

                events = events_response["data"]["events"]
                for event in events:
                    if event not in device_events["events"]:
                        device_events["events"].append(event)

                pages += 1

            self.events.append(device_events)

        return self.events

    def connect(self):
        """
        Connect to Kodak Smart Home Portal and get all information needed.

        :return: None
        :exception: ``ConnectionError``
        """
        try:
            self._options()
            self._token()
            self._authentication()
            self._get_devices()
            self._get_events()

        except requests.exceptions.ConnectionError as err:
            raise ConnectionError(str(err))

    def update(self):
        """
        Update the device list and events data

        :return: True
        :rtype: bool
        :exception: ``ConnectionError``
        """
        self._get_devices()
        self._get_events()

    def disconnect(self):
        """
        Disconnect from Kodak Smart Portal

        :return: None
        :exception: ``ConnectionError``
        """
        self._http_request("GET", self.region_url.URL_LOGOUT)
        self.http_session.close()
        self.is_connected = False

    @property
    def get_devices(self):
        """
        List all registered devices in Kodak Smart Portal and its details.

        :return: all devices and information
        :exception: ``ConnectionError``
        :rtype: list
        """
        if self.is_connected:
            return self.devices

        else:
            raise ConnectionError(
                f"Kodak Smarthome API is {self.is_connected}"
            )

    @property
    def get_events(self):
        """
        Get all devices events

        :return: list of devices events
        :exception: ``ConnectionError``
        :rtype: list
        """
        if self.is_connected:
            return self.events

        else:
            raise ConnectionError(
                f"Kodak Smarthome API is {self.is_connected}"
            )

    def get_events_device(self, device_id=None):
        """
        Get all device events

        :param device_id: device id available in the device information
            ``KodakSmartHome.get_devices``
        :type device_id: str
        :return: list events
        :rtype: list
        """
        if device_id is None:
            return self.events

        else:
            if device_id in [d["device_id"] for d in self.devices]:
                device_events = list(
                    filter(lambda d: d["device_id"] == device_id, self.events)
                )

                events = device_events[0]["events"]
                return sorted(events, key=lambda e: e["created_date"])

            else:

                return None

    def _filter_event_type(
        self, device_id=None, event_type=DEVICE_EVENT_MOTION
    ):
        """
        Filter events from device by event type.

        :param device_id: device id available in the device information
            ``KodakSmartHome.get_devices``
        :param event_type: Possible events``kodaksmarthome.constants``:
            DEVICE_EVENT_MOTION, DEVICE_EVENT_SOUND, DEVICE_EVENT_BATTERY.
            Default: DEVICE_EVENT_MOTION
        :return: events type from specified device
        :rtype: list
        """
        if self.is_connected:
            device_events = self.get_events_device(device_id=device_id)

            if device_events is None:
                return None

            if device_id is None:
                motion_events = list()
                for device in device_events:
                    motion_events += list(
                        filter(
                            lambda e: e["event_type"] == event_type,
                            device["events"],
                        )
                    )

            else:
                motion_events = list(
                    filter(
                        lambda e: e["event_type"] == event_type, device_events
                    )
                )

            return motion_events

        else:
            raise ConnectionError(
                f"Kodak Smarthome API is {self.is_connected}"
            )

    def get_motion_events(self, device_id=None):
        """
        List all motion devices events from specific device sorted by
        creation date.

        :return: list of motion devices events
        :exception: ``ConnectionError``
        :rtype: list
        """
        if self.is_connected:
            events = self._filter_event_type(
                device_id=device_id, event_type=DEVICE_EVENT_MOTION
            )

            if events is None:

                return list()

            return sorted(events, key=lambda e: e["created_date"])

        else:
            raise ConnectionError(
                f"Kodak Smarthome API is {self.is_connected}"
            )

    def get_battery_events(self, device_id=None):
        """
        List all battery devices events from specific device, sorted by
        creation date.

        :return: list of battery devices events
        :exception: ``ConnectionError``
        :rtype: list
        """
        if self.is_connected:
            events = self._filter_event_type(
                device_id=device_id, event_type=DEVICE_EVENT_BATTERY
            )

            if events is None:

                return list()

            return sorted(events, key=lambda e: e["created_date"])

        else:
            raise ConnectionError(
                f"Kodak Smarthome API is {self.is_connected}"
            )

    def get_sound_events(self, device_id=None):
        """
        List all sound devices events from specific device sorted by
        creation date.

        :return: list of sound devices events
        :exception: ``ConnectionError``
        :rtype: list
        """
        if self.is_connected:
            events = self._filter_event_type(
                device_id=device_id, event_type=DEVICE_EVENT_SOUND
            )

            if events is None:

                return list()

            return sorted(events, key=lambda e: e["created_date"])

        else:
            raise ConnectionError(
                f"Kodak Smarthome API is {self.is_connected}"
            )
